/*3:*/
#line 73 "plotting.w"


#ifndef PLOTTING_HEADER_DEFINES
#define PLOTTING_HEADER_DEFINES
#include <math.h>        

#include <iostream> 
#include <exception> 
#include <stdexcept> 
#include <string> 
#include <vector> 
#include <algorithm> 
using namespace std;

#include "bitmap.h"
#include <stdio.h>       
#include "gifout.h"
#include "graphics.h"
#include "pstampr.h"
#include "psrtext.h"

/*6:*/
#line 115 "plotting.w"

class csColour{
public:
virtual string colourSystemName(void)= 0;

virtual void writeParameters(ostream&of){

of<<"Colour system: "<<colourSystemName()<<"\n";
}
};

/*:6*//*7:*/
#line 135 "plotting.w"

class csSpectralColour:public csColour{
public:



virtual double getIntensity(double waveLength)= 0;




virtual bool isMonochromatic(void)= 0;
};

/*:7*//*8:*/
#line 156 "plotting.w"

class csMonochromaticColour:public csSpectralColour{
private:
double wavelength;

public:
virtual string colourSystemName(void){
return"monochromatic spectral";
}

double getIntensity(double waveLength)
{
return(waveLength==wavelength)?1.0:0.0;
}

bool isMonochromatic(void){
return true;
}



csMonochromaticColour(void)
{
wavelength= 0.0;
}

csMonochromaticColour(double waveLength)
{
wavelength= waveLength;
}



void setWavelength(double waveLength)
{
wavelength= waveLength;
}

double getWavelength(void)
{
return wavelength;
}

void writeParameters(ostream&of){
csSpectralColour::writeParameters(of);
of<<"    Wavelength = "<<getWavelength()<<"\n";
}
};


/*:8*//*9:*/
#line 210 "plotting.w"

class csSpectralBlack:public csSpectralColour{
public:
virtual string colourSystemName(void){
return"black spectral";
}

double getIntensity(double waveLength)
{
return 0.0;
}

bool isMonochromatic(void){
return false;
}
};

/*:9*//*10:*/
#line 232 "plotting.w"

class csSpectralWhite:public csSpectralColour{
public:
virtual string colourSystemName(void){
return"white spectral";
}

double getIntensity(double waveLength)
{
return 1.0;
}

bool isMonochromatic(void){
return false;
}
};

/*:10*//*11:*/
#line 253 "plotting.w"

class csBlackBody:public csSpectralColour{
private:

double temperature;

public:
double getIntensity(double waveLength)
{
return 0.0;
}

bool isMonochromatic(void){
return false;
}



csBlackBody(void)
{
temperature= 0.0;
}

csBlackBody(double temp)
{
temperature= temp;
}



void setTemperature(double temp)
{
temperature= temp;
}

double getTemperature(void)
{
return temperature;
}

double totalFlux(void)
{













return 5.67051e-8*
(temperature*temperature*temperature*temperature);
}

double flux(double wl)
{












return 3.74183e-16/((wl*wl*wl*wl*wl)*
(exp(1.4388e-2/(wl*temperature))-1));
}

void writeParameters(ostream&of){
csSpectralColour::writeParameters(of);
of<<"    Temperature = "<<getTemperature()<<" K\n";
}
};

/*:11*//*12:*/
#line 341 "plotting.w"

class csDeviceColour:public csColour{
public:

/*13:*/
#line 359 "plotting.w"

virtual void asRGB(double&r,double&g,double&b)= 0;
virtual void asCMYK(double&c,double&m,double&y,double&k)= 0;
virtual void asGreyScale(double&g)= 0;

/*:13*/
#line 345 "plotting.w"

/*14:*/
#line 372 "plotting.w"

virtual void asHSV(double&h,double&s,double&v);
virtual void asHLS(double&h,double&l,double&s);
virtual void asYIQ(double&y,double&i,double&q);
virtual void asYUV(double&y,double&u,double&v);
virtual void asSMPTE(double&y,double&Pb,double&Pr);
virtual void asCMY(double&c,double&m,double&y);

/*:14*/
#line 346 "plotting.w"

/*15:*/
#line 386 "plotting.w"

protected:
static double hlsval(double n1,double n2,double hue);

public:
static void hsv_rgb(double h,double s,double v,
double*r,double*g,double*b);
static void rgb_hsv(double r,double g,double b,
double*h,double*s,double*v);
static void rgb_hls(double r,double g,double b,
double*h,double*l,double*s);
static void hls_rgb(double h,double l,double s,
double*r,double*g,double*b);
static void rgb_yiq(double r,double g,double b,
double*y,double*i,double*q);
static void yiq_rgb(double y,double i,double q,
double*r,double*g,double*b);
static void rgb_yuv(double r,double g,double b,
double*y,double*u,double*v);
static void yuv_rgb(double y,double u,double v,
double*r,double*g,double*b);
static void rgb_smpte_204M(double r,double g,double b,
double*y,double*Pb,double*Pr);
static void smpte_204M_rgb(double y,double Pb,double Pr,
double*r,double*g,double*b);
static void rgb_cmy(double r,double g,double b,
double*c,double*m,double*y);
static void cmy_rgb(double c,double m,double y,
double*r,double*g,double*b);
static void cmy_cmyk(double c,double m,double y,
double*oc,double*om,double*oy,double*ok);
static void cmyk_cmy(double c,double m,double y,double k,
double*oc,double*om,double*oy);

/*:15*/
#line 347 "plotting.w"

};

/*:12*/
#line 94 "plotting.w"

#endif

/*:3*/
